/* utime -- set the file modification time of the given file
 * according to the time given; a time of 0 means the current
 * time.
 *
 * All times are in Unix format, i.e. seconds since to
 * midnight, January 1, 1970 GMT
 *
 * written by Eric R. Smith, and placed in the public domain.
 *
 */
 
#include <compiler.h>
#include <limits.h>
#include <time.h>
#include <errno.h>
#include <osbind.h>
#include <mintbind.h>
#include <assert.h>
#include <unistd.h>

#ifdef __TURBOC__
# include <sys\ioctl.h>
# include <sys\types.h>
#else
# include <sys/ioctl.h>
# include <sys/types.h>
#endif

#include "lib.h"

int
__utime(_filename, tset)
      const char *_filename;
      const struct utimbuf *tset;
{
	int fh;
	long ret;
	unsigned long actime, modtime;
	unsigned long dtime;	/* dos time equivalent */
	
	char filenamebuf[PATH_MAX];
	char* filename = (char*) _filename;
	
	struct _mutimbuf settime;
	long res;

	if (tset)
	{
		modtime = __dostime (tset->modtime);
		actime = __dostime (tset->actime);
	}
	else
	{
		actime = ((long) Tgettime () << 16) | (Tgetdate () & 0xFFFF);
		modtime = actime;
	}

	if (!__libc_unix_names)
	  {
	    filename = filenamebuf;
	    (void)_unx2dos(_filename, filename, sizeof (filenamebuf));
	  }
	settime.actime = (unsigned short) ((actime >> 16) & 0xFFFF);
	settime.acdate = (unsigned short) (actime & 0xFFFF);
	settime.modtime = (unsigned short) ((modtime >> 16) & 0xFFFF);
	settime.moddate = (unsigned short) (modtime & 0xFFFF);
	res = -EINVAL;
	
	if (tset)
		res = Dcntl(FUTIME, (long) filename, (long) &settime);
	else
		res = Dcntl(FUTIME, (long) filename, (long) 0);

	if (res != -EINVAL) {
		if (res < 0) {
			if ((res == -EPATH) && (_enoent(filename)))
				res = -ENOENT;
			errno = (int) -res;
			return -1;
		}
		return 0;
	}
	fh = (int) Fopen(filename, 2);
	if (fh < 0) {
#if 1
		/* Kludge:  on dos filesystems return success for dirs
		   even though we failed */
		if ((fh == -ENOENT) &&
		    (((ret = Dpathconf(filename, 5)) == 2) || (ret == -EINVAL)) &&
		    (Fattrib(filename, 0, 0) == FA_DIR))
			return 0;
#endif
		if ((fh == -EPATH) && (_enoent(filename)))
			fh = -ENOENT;
		errno = -fh;
		return -1;
	}

	if (tset)
		res = Fcntl(fh, (long)&settime, FUTIME);
	else
		res = Fcntl(fh, (long)0, FUTIME);

	if (res == -EINVAL)
	      {
		dtime = modtime;
		(void)Fdatime((_DOSTIME *) &dtime, fh, 1);
	      }

	if ((fh = Fclose(fh)) != 0) {
		errno = -fh;
		return -1;
	}
	return 0;
}

#ifdef HAVE_WEAK_SYMBOLS
weak_alias (__utime, utime)
#endif
